"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ElementSlashQuery = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactDom = require("react-dom");

var _core = require("@sheets-editor/core");

var _slate = require("slate");

var _slateReact = require("slate-react");

var _isHotkey = _interopRequireDefault(require("is-hotkey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const ElementSlashQuery = ({
  attributes,
  children,
  element,
  menuItems
}) => {
  const {
    search,
    result
  } = (0, _core.useSearch)(menuItems);
  const editor = (0, _slateReact.useEditor)();
  const selected = (0, _slateReact.useSelected)();
  const [activeIndex, setActiveIndexState] = (0, _react.useState)(0);
  const activeItemRef = (0, _react.useRef)(menuItems[0]);
  const elementRef = (0, _react.useRef)(element);
  const activeIndexRef = (0, _react.useRef)(0);
  const [ignoreMouse, setIgnoreMouse] = (0, _react.useState)(true);
  const [open, setOpen] = (0, _react.useState)(true);
  const [refReady, setRefReady] = (0, _react.useState)(false);
  const {
    Popover,
    List,
    MenuItem,
    MenuSectionHeading,
    MenuDivider
  } = (0, _core.useUI)();
  const ref = (0, _react.useRef)(null);

  const text = _slate.Node.string(element);

  const textRef = (0, _react.useRef)('/');
  const query = text.slice(1, -1);

  const setActiveIndex = value => {
    setActiveIndexState(value);
    activeIndexRef.current = value;
  };

  const close = (0, _react.useCallback)(() => {
    setOpen(false);
  }, []);
  const handleSubmit = (0, _react.useCallback)(() => {
    const blockAbove = (0, _core.getBlockAbove)(editor);

    if (activeItemRef.current.inline) {
      _slate.Transforms.select(editor, _slateReact.ReactEditor.findPath(editor, elementRef.current));

      editor.insertElement(activeItemRef.current.id);
    } else if (_slate.Node.string(blockAbove[0]) === textRef.current) {
      editor.turnIntoElement(activeItemRef.current.id, { ...elementRef.current,
        children: [{
          text: ''
        }]
      });

      _slate.Transforms.select(editor, blockAbove[1]);

      _slate.Transforms.delete(editor);

      setTimeout(() => {
        _slateReact.ReactEditor.focus(editor);

        _slate.Transforms.select(editor, blockAbove[1]);
      });
    } else {
      _slate.Transforms.select(editor, _slateReact.ReactEditor.findPath(editor, elementRef.current));

      _slate.Transforms.delete(editor);

      setTimeout(() => {
        editor.insertElement(activeItemRef.current.id);
      });
    }

    close();
  }, []);
  const handleKeyDown = (0, _react.useCallback)(event => {
    setIgnoreMouse(true);

    if ((0, _isHotkey.default)('Enter', event)) {
      event.preventDefault();
      handleSubmit();
    } else if ((0, _isHotkey.default)('Escape', event)) {
      close();
    } else if ((0, _isHotkey.default)('ArrowDown', event)) {
      event.preventDefault();
      setActiveIndex(activeIndexRef.current + 1);
    } else if ((0, _isHotkey.default)('ArrowUp', event)) {
      event.preventDefault();
      setActiveIndex(activeIndexRef.current - 1);
    }
  }, []);
  const handleMouseMove = (0, _react.useCallback)(() => {
    setIgnoreMouse(false);
  }, []);
  (0, _react.useEffect)(() => {
    let activeItem;

    if (activeIndex < 0) {
      setActiveIndex(result.length - 1);
      activeItem = result[result.length - 1];
    } else if (activeIndex > result.length - 1) {
      setActiveIndex(0);
      activeItem = result[0];
    } else {
      activeItem = result[activeIndex];
    }

    if (activeItem) {
      const menuItem = document.getElementById(`slash-menu-item-${activeItem.id}`);

      if (ignoreMouse && menuItem) {
        menuItem.scrollIntoView({
          block: 'nearest'
        });
      }
    }

    activeItemRef.current = activeItem;
  }, [activeIndex, result, ignoreMouse]);
  (0, _react.useEffect)(() => {
    document.addEventListener('keydown', handleKeyDown);
    document.addEventListener('mousemove', handleMouseMove);
    return () => {
      document.removeEventListener('keydown', handleKeyDown);
      document.removeEventListener('mousemove', handleMouseMove);
    };
  }, []);
  (0, _react.useEffect)(() => {
    elementRef.current = element;
  }, [element]);
  (0, _react.useEffect)(() => {
    if (!selected || query.length > 9 || text.length < 2) {
      close();
    } else {
      textRef.current = text;
      search(query);
    }
  }, [query, text]);
  (0, _react.useEffect)(() => {
    if (ref.current) {
      setRefReady(true);
    }
  }, [ref.current]);
  (0, _react.useEffect)(() => {
    if (!open) {
      const path = _slateReact.ReactEditor.findPath(editor, element);

      _slate.Transforms.unwrapNodes(editor, {
        at: path
      });
    }
  }, [open]);
  return _react.default.createElement(_react.default.Fragment, null, _react.default.createElement("span", attributes, _react.default.createElement("span", {
    contentEditable: false,
    ref: ref
  }), _react.default.createElement("span", null, children)), (0, _reactDom.createPortal)(_react.default.createElement(Popover, {
    open: refReady && open,
    onClose: close,
    anchorEl: ref.current,
    anchorOrigin: {
      horizontal: 'left',
      vertical: 'top'
    },
    transformOrigin: {
      horizontal: 'left',
      vertical: 'bottom'
    }
  }, _react.default.createElement("div", {
    style: {
      maxHeight: '40vh',
      overflow: 'hidden auto',
      maxWidth: 'calc(100vw - 32px)',
      width: 330
    }
  }, _react.default.createElement(List, null, result.map((item, index) => _react.default.createElement(_react.Fragment, {
    key: item.id
  }, (index === 0 || item.group !== result[index - 1].group) && _react.default.createElement(_react.default.Fragment, null, index !== 0 && _react.default.createElement(MenuDivider, null), _react.default.createElement(MenuSectionHeading, null, item.group)), _react.default.createElement("div", {
    id: `slash-menu-item-${item.id}`
  }, _react.default.createElement(MenuItem, {
    onMouseEnter: () => setActiveIndex(index),
    onClick: event => {
      event.preventDefault();
      event.stopPropagation();
      handleSubmit();
    },
    style: index === activeIndex ? {
      backgroundColor: '#DEEAFF'
    } : {},
    primaryText: item.title,
    secondaryText: item.subtitle,
    image: item.image,
    imageSize: item.imageSize || 'large',
    icon: item.icon,
    tooltip: item.tooltip,
    tooltipImage: item.tooltipImage
  })))), result.length === 0 && _react.default.createElement("div", {
    style: {
      padding: '4px 16px',
      color: '#878682',
      fontSize: '14px'
    }
  }, "No matches")), ignoreMouse && _react.default.createElement("div", {
    style: {
      position: 'absolute',
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    }
  }))), document.body));
};

exports.ElementSlashQuery = ElementSlashQuery;
//# sourceMappingURL=ElementSlashQuery.js.map